/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.interpreter;

import alefpp.core.ALast;
import alefpp.core.ANext;
import alefpp.core.ARet;
import alefpp.core.exceptions.ConversionTypeException;
import alefpp.core.exceptions.DuplicateLocalVariableException;
import alefpp.core.exceptions.DuplicateSubroutineException;
import alefpp.core.exceptions.FinalLocalVariableException;
import alefpp.core.exceptions.ModuleCastException;
import alefpp.core.exceptions.OutOfParametersException;
import alefpp.core.exceptions.PackageNotFoundException;
import alefpp.core.exceptions.UndefinedFieldException;
import alefpp.core.exceptions.UndefinedObjectException;
import alefpp.core.exceptions.UndefinedSubroutineException;
import alefpp.core.exceptions.UndefinedTypeException;
import alefpp.core.exceptions.UnsupportedFeatureException;
import alefpp.core.exceptions.VariableNotFoundException;
import alefpp.core.exceptions.VoidSubroutinesException;
import alefpp.parser.syntaxtree.*;

/**
 * 
 * @author Adrabi Abderrahim
 * @version 1.27
 */

@SuppressWarnings("all")
public interface Interpreter {

   //
   // User-generated visitor methods below
   //

   /**
    * f0 -> ( UseStatement() )*
    * f1 -> ( EmptyStatement() | VariableDeclaration() ";" | SubroutineDeclaration() | Module() )*
    * f2 -> ( Main() )?
    * f3 -> <EOF>
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws PackageNotFoundException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws DuplicateSubroutineException 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(Alefpp $1 , String[] $2) throws ConversionTypeException, DuplicateLocalVariableException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, PackageNotFoundException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, DuplicateSubroutineException, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "use"
    * f1 -> <IDENTIFIER>
    * f2 -> ( "." ( <IDENTIFIER> | "*" ) )*
    * f3 -> [ ":" <STRING_LITERAL> ]
    * f4 -> ";"
    * @throws PackageNotFoundException 
    */
   public Object visit(UseStatement $1 , Object $2, Object ... $3) throws PackageNotFoundException;

   /**
    * f0 -> <SPCIAL_SCALAR>
    *       | <SCALAR>
    *       | "$"
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
    */
   public Object visit(Scalar $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException;

   /**
    * f0 -> "<-"
    * f1 -> Expression()
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast 
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ScalarAssign $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "my"
    * f1 -> [ "`" ]
    * f2 -> <SCALAR>
    * f3 -> [ FullIdentifier() ]
    * f4 -> ( "@" | "%" )*
    * f5 -> [ ScalarAssign() ]
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(VariableDeclaration $1 , Object $2, Object ... $3) throws ConversionTypeException, DuplicateLocalVariableException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "["
    * f1 -> [ ArgumentList() ]
    * f2 -> "]"
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(Arguments $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> Expression()
    * f1 -> ( "," Expression() )*
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ArgumentList $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "#"
    * f1 -> ( ModuleSubroutineCalling() | <INTEGER_LITERAL> | Expression() )
 * @throws UnsupportedFeatureException 
 * @throws ModuleCastException 
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ArrayIndex $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "\\"
    * f1 -> ( <IDENTIFIER> | ModuleSubroutineCalling() | Expression() ) 
    * @throws Throwable 
    */
   public Object visit(HashKey $1 , Object $2, Object ... $3) throws Throwable;

   /**
    * f0 -> "["
    * f1 -> ( Expression() ( "," Expression() )* )?
    * f2 -> "]"
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(Array $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "{"
    * f1 -> ( <IDENTIFIER> "=>" Expression() ( "," <IDENTIFIER> "=>" Expression() )* )?
    * f2 -> "}"
    * @throws Throwable 
    */
   public Object visit(Hash $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, Throwable;

   /**
    * f0 -> "->"
    * f1 -> ( <IDENTIFIER> [ Arguments() ] | Scalar() [ ArrayIndex() | HashKey() ] )
    * f2 -> ( MemberAccess() )*
    */
   public Object visit(MemberAccess $1 , Object $2, Object ... $3);

   /**
    * f0 -> <IDENTIFIER>
    * f1 -> ( "." <IDENTIFIER> )*
    */
   public Object visit(FullIdentifier $1 , Object $2, Object ... $3);

   /**
    * f0 -> FullIdentifier()
    * f1 -> ( ( "::" )? Arguments() )?
    * f2 -> [ MemberAccess() ]
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ModuleSubroutineCalling $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> CastStatement()
    *       | ReferenceStatement()
    *       | XiSubroutine()
    *       | ConditionalExpression()
    *       | ConditionalOrExpression()
    *       | AdditiveExpression()
    *       | MultiplicativeExpression()
    *       | BitwiseExpression()
    *       | ShiftExpression()
    *       | Array()
    *       | Hash()
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(Expression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "to"
    * f1 -> Expression()
    * f2 -> FullIdentifier()
 * @throws UnsupportedFeatureException 
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(CastStatement $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "ref"
    * f1 -> Expression()
    * f2 -> FullIdentifier()
 * @throws UnsupportedFeatureException 
 * @throws Throwable 
    */
   public Object visit(ReferenceStatement $1 , Object $2, Object ... $3) throws Throwable;

   /**
    * f0 -> ( "+" | "-" )
    * f1 -> ( Expression() )+
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(AdditiveExpression $1 , Object $2, Object ... $3) throws  VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ( "*" | "/" | "%" | "//" | "**" )
    * f1 -> ( Expression() )+
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(MultiplicativeExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ( "&" | "|" )
    * f1 -> ( Expression() )+
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(BitwiseExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ( "<<" | ">>" )
    * f1 -> ( Expression() )+
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ShiftExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ConditionalAndExpression()
    * f1 -> ( "||" ConditionalAndExpression() )*
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ConditionalOrExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> EqualityExpression()
    * f1 -> ( "&&" EqualityExpression() )*
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ConditionalAndExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> RelationalExpression()
    * f1 -> ( ( "==" | "!=" ) RelationalExpression() )*
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(EqualityExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ExclusiveOrExpression()
    * f1 -> ( ( "<" | ">" | "<=" | ">=" ) ExclusiveOrExpression() )*
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(RelationalExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ValuesExpression()
    * f1 -> ( "^" ValuesExpression() )*
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ExclusiveOrExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "walo"
    */
   public Object visit(WaloExpression $1 , Object $2, Object ... $3);

   /**
    * f0 -> WaloExpression()
    *       | <STRING_LITERAL>
    *       | <INTEGER_LITERAL>
    *       | <FLOATING_POINT_LITERAL>
    *       | PreInDeCrementExpression()
    *       | InDeCrementExpression()
    *       | ScalarValue() [ MemberAccess() ] [ ScalarAssign() ]
    *       | ModuleSubroutineCalling() [ ScalarAssign() ]
    *       | "(" Expression() ")"
    *       | "!" Expression()
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ValuesExpression $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ( "++" | "--" )
    * f1 -> ( ScalarValue() [ MemberAccess() ] | ModuleSubroutineCalling() )
 * @throws UnsupportedFeatureException 
 * @throws Throwable 
    */
   public Object visit(PreInDeCrementExpression $1 , Object $2, Object ... $3) throws Throwable;

   /**
    * f0 -> ( ScalarValue() [ MemberAccess() ] | ModuleSubroutineCalling() )
    * f1 -> ( "++" | "--" )
 * @throws UnsupportedFeatureException 
 * @throws ARet 
 * @throws ANext 
 * @throws ALast 
 * @throws VoidSubroutinesException 
 * @throws UndefinedObjectException 
 * @throws OutOfParametersException 
 * @throws DuplicateLocalVariableException 
 * @throws ModuleCastException 
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws Throwable 
    */
   public Object visit(InDeCrementExpression $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, UndefinedObjectException, VoidSubroutinesException, ALast, ANext, ARet, Throwable;

   /**
    * f0 -> "="
    * f1 -> Expression()
    * f2 -> ","
    * f3 -> ValuesExpression()
    * f4 -> [ "," ValuesExpression() ]
 * @throws Throwable 
    */
   public Object visit(ConditionalExpression $1 , Object $2, Object ... $3) throws  Throwable;

   /**
    * f0 -> Scalar()
    * f1 -> ( ArrayIndex() | HashKey() )*
 * @throws ConversionTypeException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws ModuleCastException 
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws DuplicateLocalVariableException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ScalarValue $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "eval"
    * f1 -> BlockStatement()
    * f2 -> ( "catch" "(" ( <SCALAR> | "my" <SCALAR> [ FullIdentifier() ] ) ")" BlockStatement() )+
    * f3 -> [ "end" BlockStatement() ]
 * @throws UnsupportedFeatureException 
 * @throws Throwable 
    */
   public Object visit(EvalStatement $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, Throwable;

   /**
    * f0 -> "goto"
    * f1 -> Expression()
    * f2 -> ":"
    * f3 -> BlockStatement()
 * @throws UnsupportedFeatureException 
 * @throws Throwable 
    */
   public Object visit(GotoStatement $1 , Object $2, Object ... $3) throws  Throwable;

   /**
    * f0 -> "miss"
    * f1 -> ":"
    * f2 -> BlockStatement()
 * @throws UnsupportedFeatureException 
 * @throws Throwable 
    */
   public Object visit(MissStatement $1 , Object $2, Object ... $3) throws  Throwable;

   /**
    * f0 -> "jump"
    * f1 -> Expression()
    * f2 -> ( GotoStatement() )+
    * f3 -> [ MissStatement() ]
    * @throws Throwable 
    */
   public Object visit(JumpStatement $1 , Object $2, Object ... $3) throws Throwable;

   /**
    * f0 -> Expression()
    * f1 -> "!loop"
    * f2 -> BlockStatement()
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws DuplicateLocalVariableException 
 * @throws ConversionTypeException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(DoWhileStatement $1 , Object $2, Object ... $3) throws ConversionTypeException, DuplicateLocalVariableException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, OutOfParametersException, ALast, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> Expression()
    * f1 -> "loop"
    * f2 -> BlockStatement()
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(WhileStatement $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, DuplicateLocalVariableException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> EachInit()
    * f1 -> ","
    * f2 -> Expression()
    * f3 -> "each"
    * f4 -> BlockStatement()
 * @throws Throwable 
    */
   public Object visit(EachStatement $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, Throwable;

   /**
    * f0 -> EachInit()
    * f1 -> ","
    * f2 -> Expression()
    * f3 -> "go"
    * f4 -> [ "(" Expression() ")" ]
    * f5 -> BlockStatement()
 * @throws Throwable 
 * @throws UnsupportedFeatureException 
    */
   // public Object visit(GoStatement $1 , Object $2, Object ... $3) throws UnsupportedFeatureException;
   public Object visit(GoStatement $1 , Object $2, Object ... $3) throws Throwable;

   /**
    * f0 -> "my" <SCALAR> [ FullIdentifier() ] [ ScalarAssign() ]
    *       | PreInDeCrementExpression()
    *       | InDeCrementExpression()
    *       | ModuleSubroutineCalling() [ ScalarAssign() ]
    *       | ScalarValue() [ MemberAccess() ] [ ScalarAssign() ]
 * @throws Throwable 
    */
   public Object visit(GoInit $1 , Object $2, Object ... $3) throws Throwable;
   
   /**
    * f0 -> Expression()
    * f1 -> "=>"
    * f2 -> BlockStatement()
    * f3 -> [ "," BlockStatement() ]
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(IfStatement $1 , Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, DuplicateLocalVariableException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "my" <SCALAR> [ FullIdentifier() ]
    *       | ModuleSubroutineCalling()
    *       | ScalarValue() [ MemberAccess() ] 
    * @throws DuplicateLocalVariableException 
    * @throws Throwable 
    */
   public Object visit(EachInit $1 , Object $2, Object ... $3) throws  UndefinedTypeException, DuplicateLocalVariableException, Throwable;

   /**
    * f0 -> <IDENTIFIER>
    * f1 -> ":"
 * @throws UnsupportedFeatureException 
    */
   public Object visit(Label $1 , Object $2, Object ... $3) throws UnsupportedFeatureException;

   /**
    * f0 -> "next"
    * f1 -> [ <IDENTIFIER> ]
 * @throws ANext 
    */
   public Object visit(Next $1 , Object $2, Object ... $3) throws ANext;

   /**
    * f0 -> "last"
    * f1 -> [ <IDENTIFIER> ]
 * @throws ALast 
    */
   public Object visit(Last $1 , Object $2, Object ... $3) throws ALast;

   /**
    * f0 -> FullIdentifier()
    * f1 -> [ ( "[" )+ | ( "@" | "%" )+ ]
 * @throws UnsupportedFeatureException 
    */
   public Object visit(ParameterType $1 , Object $2, Object ... $3) throws UnsupportedFeatureException;

   /**
    * f0 -> ":"
    * f1 -> FullIdentifier()
    * f2 -> ( ( "[" )+ | ( "@" | "%" )+ )?
 * @throws UnsupportedFeatureException 
    */
   public Object visit(SubroutineType $1 , Object $2, Object ... $3) throws UnsupportedFeatureException;

   /**
    * f0 -> "sub"
    * f1 -> <IDENTIFIER>
    * f2 -> [ SubroutineType() ]
    * f3 -> ( "(" [ ParameterType() ( "," ParameterType() )* ] ")" )?
    * f4 -> [ DiesStatement() ]
    * f5 -> Block()
 * @throws UnsupportedFeatureException 
 * @throws DuplicateSubroutineException 
    */
   public Object visit(SubroutineDeclaration $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, DuplicateSubroutineException;

   /**
    * f0 -> Block()
    * f1 -> Arguments()
 * @throws UnsupportedFeatureException 
 * @throws ARet 
 * @throws ANext 
 * @throws ALast 
 * @throws VoidSubroutinesException 
 * @throws UndefinedObjectException 
 * @throws OutOfParametersException 
 * @throws DuplicateLocalVariableException 
 * @throws ModuleCastException 
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws Throwable 
    */
   public Object visit(XiSubroutine $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, UndefinedObjectException, VoidSubroutinesException, ALast, ANext, ARet, Throwable;

   /**
    * f0 -> "ret"
    * f1 -> Expression()
    * f2 -> ";"
 * @throws UnsupportedFeatureException 
 * @throws OutOfParametersException 
 * @throws DuplicateLocalVariableException 
 * @throws ModuleCastException 
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(ReturnStatement $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "die"
    * f1 -> Expression()
 * @throws UnsupportedFeatureException 
 * @throws ARet 
 * @throws ANext 
 * @throws ALast 
 * @throws VoidSubroutinesException 
 * @throws UndefinedObjectException 
 * @throws OutOfParametersException 
 * @throws DuplicateLocalVariableException 
 * @throws ModuleCastException 
 * @throws UndefinedFieldException 
 * @throws UndefinedTypeException 
 * @throws UndefinedSubroutineException 
 * @throws FinalLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws ConversionTypeException 
 * @throws Throwable 
    */
   public Object visit(DieStatement $1 , Object $2, Object ... $3) throws UnsupportedFeatureException, ConversionTypeException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, DuplicateLocalVariableException, OutOfParametersException, UndefinedObjectException, VoidSubroutinesException, ALast, ANext, ARet, Throwable;

   /**
    * f0 -> "synchro"
    * f1 -> "("
    * f2 -> Expression()
    * f3 -> ")"
    * f4 -> Block()
 * @throws UnsupportedFeatureException 
 * @throws Throwable 
    */
   public Object visit(SynchonizedStatement $1 , Object $2, Object ... $3) throws  Throwable;

   /**
    * f0 -> "main"
    * f1 -> [ DiesStatement() ]
    * f2 -> Block()
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(Main $1 , Object $2, Object ... $3) throws ConversionTypeException, DuplicateLocalVariableException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> "{"
    * f1 -> ( BlockStatement() )*
    * f2 -> "}"
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(Block $1 , Object $2, Object ... $3) throws ConversionTypeException, DuplicateLocalVariableException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> EmptyStatement()
    *       | Block()
    *       | Statement()
    *       | ReturnStatement()
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(BlockStatement $1 , Object $2, Object ... $3) throws ConversionTypeException, DuplicateLocalVariableException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> DoWhileStatement()
    *       | WhileStatement()
    *       | EachStatement()
    *       | GoStatement()
    *       | AIfStatement()
    *       | Label()
    *       | Last() ";"
    *       | $1ext() ";"
    *       | VariableDeclaration() ";"
    *       | PreInDeCrementExpression() ";"
    *       | InDeCrementExpression() ";"
    *       | ScalarValue() [ MemberAccess() ] [ ScalarAssign() ] ";"
    *       | ModuleSubroutineCalling() [ ScalarAssign() ] ";"
    *       | DieStatement() ";"
    *       | JumpStatement() ";"
    *       | EvalStatement()
    *       | SynchonizedStatement()
 * @throws ConversionTypeException 
 * @throws DuplicateLocalVariableException 
 * @throws VariableNotFoundException 
 * @throws FinalLocalVariableException 
 * @throws UndefinedSubroutineException 
 * @throws UndefinedTypeException 
 * @throws UndefinedFieldException 
 * @throws ModuleCastException 
 * @throws OutOfParametersException
 * @throws ALast
 * @throws ANext 
 * @throws ARet 
 * @throws UndefinedObjectException 
 * @throws VoidSubroutinesException 
 * @throws Throwable 
    */
   public Object visit(Statement $1 , Object $2, Object ... $3) throws ConversionTypeException, DuplicateLocalVariableException, VariableNotFoundException, FinalLocalVariableException, UndefinedSubroutineException, UndefinedTypeException, UndefinedFieldException, ModuleCastException, OutOfParametersException, ALast, ANext, ARet, UndefinedObjectException, VoidSubroutinesException, Throwable;

   /**
    * f0 -> ( Modifier() )*
    * f1 -> "module"
    * f2 -> FullIdentifier()
    * f3 -> ModuleBodyBlock()
 * @throws UnsupportedFeatureException 
    */
   public Object visit(Module $1 , Object $2, Object ... $3) throws UnsupportedFeatureException;

}

